﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using FlashCard7BL;
using FlashCard7BL.Interficies;

namespace FlashCard7
{
    /// <summary>
    /// Interaction logic for TargetaControl.xaml
    /// </summary>
    public partial class TargetaControl : UserControl
    {

        #region Propietats

        public enum TipusTargeta
        {
            Central = 0,
            Esquerra = 1,
            Dreta = 2
        }

        private Targeta _targeta;
        private bool _esDavant;
        private TipusTargeta _tipus;
        private const string CaraDavant = "A";
        private const string CaraDarrere = "B";

        public Targeta Targeta
        {
            get { return _targeta; }
            set { _targeta = value; }
        }

        public bool EsDavant
        {
            get { return _esDavant; }
            set { _esDavant = value; }
        }

        public TipusTargeta Tipus
        {
            get { return _tipus; }
            set { _tipus = value; }
        }

        #endregion

        #region Constructors

        public TargetaControl()
        {
            InitializeComponent();
            this.Margin = new Thickness(0);
            this.Height = 170;
            this.Width = 270;
        }


        #endregion

        #region Events

        #endregion

        #region Metodes

        public void Gira()
        {
            this.EsDavant = !this.EsDavant;

            if (this.Targeta != null)
            {
                if (EsDavant)
                {
                    this.MostraDavant();
                }
                else
                {
                    this.MostraDarrere();
                }
            }
        }

        private void MostraDavant()
        {
            this.txbTitol.Text = this._targeta.TitolFront;
            if (this._targeta.DescFront != null)
            {
                this.txbDescripcio.Text = this._targeta.DescFront;
            }
            else
            {
                this.txbDescripcio.Text = "";
            }
            this.txbCara.HorizontalAlignment = System.Windows.HorizontalAlignment.Right;
            this.txbCara.Text = CaraDavant;
        }

        private void MostraDarrere()
        {
            this.txbTitol.Text = this._targeta.TitolBack;
            if (this._targeta.DescBack != null)
            {
                this.txbDescripcio.Text = this._targeta.DescBack;
            }
            else
            {
                this.txbDescripcio.Text = "";
            }
            this.txbCara.HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
            this.txbCara.Text = CaraDarrere;
        }

        /// <summary>
        /// Depenent del tipus de targeta que és es mostra i actúa d'una manera diferent 
        /// </summary>
        /// <param name="t"></param>
        public void Carrega(Targeta t)
        {
            this.Targeta = t;
            switch (this.Tipus)
            {
                case TipusTargeta.Central:
                    if (this.Targeta != null)
                    {
                        this.EsDavant = true;
                        this.MostraDavant();
                        this.Mostra();
                    }
                    else
                    {
                        this.Amaga();
                    }
                    break;
                case TipusTargeta.Esquerra:
                    if (this.Targeta != null)
                    {
                        this.MostraDarrere();
                        this.Mostra();
                    }
                    else
                    {
                        this.Amaga();
                    }
                    break;
                case TipusTargeta.Dreta:
                    if (this.Targeta != null)
                    {
                        this.MostraDavant();
                        this.Mostra();
                    }
                    else
                    {
                        this.Amaga();
                    }
                    break;
                default:
                    break;
            }
        }

        public void Amaga()
        {
            this.Visibility = System.Windows.Visibility.Collapsed;
        }

        public void Mostra()
        {
            this.Visibility = System.Windows.Visibility.Visible;
        }

        #endregion

    }
}
