﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using FlashCard7BL;
using FlashCard7BL.Interficies;

namespace FlashCard7
{
    /// <summary>
    /// Interaction logic for UsersManagementWindow.xaml
    /// </summary>
    public partial class UsersManagementWindow : Window
    {
        BLClientProxy proxy = new BLClientProxy();

        private Usuari _usuariEditant;

        public Usuari UsuariEditant
        {
            get { return _usuariEditant; }
            set { _usuariEditant = value; }
        }

        public UsersManagementWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {

            this.cmbRol.Items.Clear();
            foreach (Rol r in proxy.GetRols())
            {
                this.cmbRol.Items.Add(r);
            }
            if (this.cmbRol.Items.Count > 0)
            {
                this.cmbRol.SelectedIndex = 0;
            }

            this.txtNom.Focus();

            if (this.UsuariEditant.Id != 0)
            {
                this.OmpleDetalls();
            }
        }

        private void OmpleDetalls()
        {
            txtCognom1.Text = this.UsuariEditant.Cognom1;
            txtCognom2.Text = this.UsuariEditant.Cognom2;
            txtPassword.Text = this.UsuariEditant.Contrasenya;
            txtCredit.Text = this.UsuariEditant.Credit.ToString();
            txtEmail.Text = this.UsuariEditant.Email;
            txtLogin.Text = this.UsuariEditant.Login;
            txtNom.Text = this.UsuariEditant.Nom;
            cmbRol.SelectedItem = this.UsuariEditant.RolE;
            
        }

        private void btnAccepta_Click(object sender, RoutedEventArgs e)
        {
            if (this.UsuariEditant != null)
            {
                if(!this.ComprovaCamps())
                {
                    MessageBox.Show(Properties.Resources.rxFaltenDades, Properties.Resources.rxAvis, MessageBoxButton.OK, MessageBoxImage.Warning);
                }
                else
                {
                    if (this.AplicaCanvis())
                    {
                        if (proxy.InsertOrEditUsuari(this.UsuariEditant))
                        {
                            if (this.UsuariEditant.Id != 0)
                            {
                                MessageBox.Show(Properties.Resources.rxUsuariActualitzat, Properties.Resources.rxCanvisSatisfactoris, MessageBoxButton.OK, MessageBoxImage.Information);
                                this.DialogResult = true;
                            }
                            else
                            {
                                MessageBox.Show(Properties.Resources.rxUsuariAfegit, Properties.Resources.rxCanvisSatisfactoris, MessageBoxButton.OK, MessageBoxImage.Information);
                                this.DialogResult = true;
                            }
                        }
                        else
                        {
                            MessageBox.Show(Properties.Resources.rxSistemaCaigut, Properties.Resources.rxAvis, MessageBoxButton.OK, MessageBoxImage.Warning);
                        }
                    }
                }
            }
        }

        private void btnCancela_Click(object sender, RoutedEventArgs e)
        {
            this.DialogResult = false;
        }

        private bool AplicaCanvis()
        {
            try
            {
                if (cmbRol.SelectedItem is Rol)
                {
                    this.UsuariEditant.RolE = (Rol)cmbRol.SelectedItem;
                }
                else
                {
                    return false;
                }

                this.UsuariEditant.Cognom1 = txtCognom1.Text.Trim();
                this.UsuariEditant.Cognom2 = txtCognom2.Text.Trim();
                this.UsuariEditant.Contrasenya = txtPassword.Text.Trim();
                this.UsuariEditant.Credit = decimal.Parse(txtCredit.Text.Trim());
                this.UsuariEditant.Email = txtEmail.Text.Trim();
                this.UsuariEditant.Login = txtLogin.Text.Trim();
                this.UsuariEditant.Nom = txtNom.Text.Trim();

                return true;
            }
            catch
            {
                return false;
            }
        }

        private bool ComprovaCamps()
        {
            if (txtNom.Text == string.Empty)
            {
                txtNom.Focus();
                return false;
            }

            if (txtLogin.Text == string.Empty)
            {
                txtLogin.Focus();
                return false;
            }

            if (txtPassword.Text == string.Empty)
            {
                txtPassword.Focus();
                return false;
            }

            if (txtCognom1.Text == string.Empty)
            {
                txtCognom1.Focus();
                return false;
            }

            if (txtEmail.Text == string.Empty)
            {
                txtEmail.Focus();
                return false;
            }

            if(txtCredit.Text == string.Empty)
            {
                txtCredit.Focus();
                return false;
            }

            decimal a;
            if (!Decimal.TryParse(txtCredit.Text, out a))
            {
                txtCredit.Focus();
                return false;
            }

            if (cmbRol.SelectedItem == null)
            {
                cmbRol.Focus();
                return false;
            }

            return true;
            
        }
    }
}
