﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Configuration;
using FlashCard7BL;
using FlashCard7BL.Interficies;
using System.ComponentModel;

namespace FlashCard7
{
    /// <summary>
    /// Interaction logic for LoginWindow.xaml
    /// </summary>
    public partial class LoginWindow : Window
    {
        BackgroundWorker worker = new BackgroundWorker();

        #region Propietats

        private string _login;
        private string _contrasenya;
        private bool _loguejat;

        public string Login
        {
            get { return _login; }
            set { _login = value; }
        }

        public string Contrasenya
        {
            get { return _contrasenya; }
            set { _contrasenya = value; }
        }

        public bool Loguejat
        {
            get { return _loguejat; }
            set { _loguejat = value; }
        }


        #endregion

        public LoginWindow()
        {
            InitializeComponent();

            // S'utilitza un Background worker per a poder mostrar la barra de càrrega mentre l'aplicació fa la transacció de Login
            worker.DoWork += new DoWorkEventHandler(worker_DoWork);
            worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(worker_RunWorkerCompleted);

            txtLogin.Focus();
        }

        #region Mètodes

        private bool ValidaCamps()
        {
            if (txtLogin.Text == string.Empty)
            {
                MessageBox.Show(Properties.Resources.rxCampLoginBuit, Properties.Resources.rxAvis, MessageBoxButton.OK, MessageBoxImage.Warning);
                txtLogin.Focus();
                return false;
            }
            else
            {
                if (txtContrasenya.Password == string.Empty)
                {
                    MessageBox.Show(Properties.Resources.rxCampContrasenyaBuit, Properties.Resources.rxAvis, MessageBoxButton.OK, MessageBoxImage.Warning);
                    txtContrasenya.Focus();
                    return false;
                }
                else
                {
                    return true;
                }
            }
        }

        private bool ValidaUsuari(string sLogin, string sContrasenya)
        {
            BLClientProxy proxy = new BLClientProxy();

            try
            {
                VariablesGlobals.UsuariActual = proxy.LoginUsuari(sLogin, sContrasenya);

                if (VariablesGlobals.UsuariActual == null)
                {
                    MessageBox.Show(Properties.Resources.rxLoginContrasenyaEquivocats, Properties.Resources.rxLogin, MessageBoxButton.OK, MessageBoxImage.Information);
                    return false;
                }
                else
                {
                    return true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(Properties.Resources.rxSistemaCaigut + Environment.NewLine + Properties.Resources.rxCodiError + " " + ex.Message, Properties.Resources.rxError,
                    MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }
            finally
            {
                if (proxy.State == System.ServiceModel.CommunicationState.Faulted)
                {
                    proxy.Abort();
                }
                else
                {
                    proxy.Close();
                }
            }
        }

        private void Carregant(bool esCarrega)
        {
            btnAccept.IsEnabled = !esCarrega;
            btnCancel.IsEnabled = !esCarrega;
            if (esCarrega)
            {
                pbCarregant.Visibility = System.Windows.Visibility.Visible;
            }
            else
            {
                pbCarregant.Visibility = System.Windows.Visibility.Hidden;
            }
        }

        #endregion


        #region Events

        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            App.Current.Shutdown();
        }

        private void btnAccept_Click(object sender, RoutedEventArgs e)
        {
            if (ValidaCamps())
            {
                Login = txtLogin.Text;
                Contrasenya = txtContrasenya.Password;

                Carregant(true);
                worker.RunWorkerAsync();
            }
        }

        void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (Loguejat)
            {
                MainWindow main = new MainWindow();
                main.Show();
                this.Close();
            }
            else
            {
                Carregant(false);
            }
        }

        void worker_DoWork(object sender, DoWorkEventArgs e)
        {
            Loguejat = ValidaUsuari(Login, Contrasenya);
        }


        #endregion
    }
}
