﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Configuration;
using FlashCard7BL;
using FlashCard7BL.Interficies;

namespace FlashCard7
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Propietats

        BLClientProxy proxy = new BLClientProxy();

        // Per a poder mantenir un control de les finestres obertes cada una té la seva instància
        UsersManagementControl usersManagementC;
        PacksManagementControl packsManagementC;
        StudyControl studyC;
        FlashCardSystemControl flashCardSystemC;
        MercatControl MercatC;

        // És necessari saber el rol de l'usuari actual per a poder fer ús dels permisos
        VariablesGlobals.RolsEnum RolUsuariActual;

        List<UserControl> llistaControls = new List<UserControl>();

        #endregion

        public MainWindow()
        {
            InitializeComponent();
        }

        #region Mètodes

        /// <summary>
        /// Amaga tots els controls que hi ha actius en la pantalla principal
        /// </summary>
        public void AmagaControls()
        {
            if (llistaControls != null)
            {
                foreach (UserControl control in llistaControls)
                {
                    control.Visibility = System.Windows.Visibility.Collapsed;
                }
            }
        }

        /// <summary>
        /// Actualitza la informació de l'usuari de la barra d'estat
        /// </summary>
        /// <returns></returns>
        public bool ActualitzaInfoUsuari()
        {
            if (VariablesGlobals.UsuariActual != null)
            {
                sbiLogin.Content = VariablesGlobals.UsuariActual.Login;
                sbiRol.Content = VariablesGlobals.UsuariActual.RolE.Nom;
                sbiCredit.Content = string.Format("{0:0.00}", VariablesGlobals.UsuariActual.Credit) + Properties.Resources.rxMoneda;
                return true;
            }

            return false;
        }

        /// <summary>
        /// Depenent dels permisos de l'usuari pot veure o no segons quins menús
        /// </summary>
        private void MostraMenus()
        {
            if (VariablesGlobals.UsuariActual != null)
            {
                RolUsuariActual = (VariablesGlobals.RolsEnum)VariablesGlobals.UsuariActual.Rol;
                if (RolUsuariActual == VariablesGlobals.RolsEnum.Administrador)
                {
                    this.mniUsuaris.Visibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    this.mniUsuaris.Visibility = System.Windows.Visibility.Collapsed;
                }
            }
        }

        #endregion

        #region Events
        private void mniSurt_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (!VariablesGlobals.ApagaVamonos)
            {
                MessageBoxResult resultat = MessageBox.Show(Properties.Resources.rxPreguntaTancarAplicació, Properties.Resources.rxSurt, MessageBoxButton.YesNo, MessageBoxImage.Question);
                e.Cancel = (resultat == MessageBoxResult.No);
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            VariablesGlobals.ApagaVamonos = false;

            if (!this.ActualitzaInfoUsuari())
            {
                MessageBox.Show(Properties.Resources.rxErrorSistema, Properties.Resources.rxError, MessageBoxButton.OK, MessageBoxImage.Error);
                VariablesGlobals.ApagaVamonos = true;
                this.Close();
            }

            VariablesGlobals.MainFinestra = this;

            this.MostraMenus();
        }

        private void mniCredit_Click(object sender, RoutedEventArgs e)
        {
            CreditWindow creditW = new CreditWindow();
            creditW.Show();
        }

        private void mniUsuaris_Click(object sender, RoutedEventArgs e)
        {
            this.AmagaControls();

            if (this.usersManagementC == null)
            {
                this.usersManagementC = new UsersManagementControl();
                this.llistaControls.Add(this.usersManagementC);
                mainPanel.Children.Add(this.usersManagementC);
            }
            else
            {
                this.usersManagementC.Visibility = Visibility.Visible;
            }
        }

        private void mniPaquets_Click(object sender, RoutedEventArgs e)
        {
            this.AmagaControls();

            if (this.packsManagementC == null)
            {
                this.packsManagementC = new PacksManagementControl();
                this.llistaControls.Add(this.packsManagementC);
                mainPanel.Children.Add(this.packsManagementC);
            }
            else
            {
                this.packsManagementC.Visibility = Visibility.Visible;
            }
        }

        private void mniMenuEstudia_Click(object sender, RoutedEventArgs e)
        {
            this.AmagaControls();

            if (this.studyC == null)
            {
                this.studyC = new StudyControl();
                this.llistaControls.Add(this.studyC);
                mainPanel.Children.Add(this.studyC);
            }
            else
            {
                this.studyC.CarregaPaquets();
                this.studyC.Visibility = Visibility.Visible;
            }
        }

        public void TargetesEstudia(Paquet paquet)
        {
            this.AmagaControls();

            if (this.flashCardSystemC == null)
            {
                this.flashCardSystemC = new FlashCardSystemControl();
                this.flashCardSystemC.CarregaPaquet(paquet);
                this.llistaControls.Add(this.flashCardSystemC);
                mainPanel.Children.Add(this.flashCardSystemC);
            }
            else
            {
                this.flashCardSystemC.CarregaPaquet(paquet);
                this.flashCardSystemC.Visibility = Visibility.Visible;
            }
        }

        private void mniMercat_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.AmagaControls();

                List<Tematica> llistaT = proxy.GetTematiques();

                if (this.MercatC == null)
                {
                    this.MercatC = new MercatControl();
                    this.MercatC.CarregaTemes(llistaT);
                    this.llistaControls.Add(this.MercatC);
                    mainPanel.Children.Add(this.MercatC);
                }
                else
                {
                    this.MercatC.CarregaTemes(llistaT);
                    this.MercatC.Visibility = Visibility.Visible;
                }
            }
            catch
            {
                this.AmagaControls();
                MessageBox.Show(Properties.Resources.rxSistemaCaigut, Properties.Resources.rxAvis, MessageBoxButton.OK, MessageBoxImage.Warning);
            }
        }
        #endregion

        private void mniAjuda_Click(object sender, RoutedEventArgs e)
        {
            //Proves proves = new Proves();
            //proves.Show();
        }

        private void mniAbout_Click(object sender, RoutedEventArgs e)
        {
            AboutBox7 about = new AboutBox7();
            about.ShowDialog();
        }

    }
}
