﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using System.Data.Linq;
using System.Data.SqlClient;
using System.Diagnostics;
using FlashCard7BL;
using FlashCard7BL.Interficies;

namespace FlashCard7DAL
{
    public class DAL : IBL
    {
        // A canviar segons el Data Source de la Base de dades
        private const string dataSource = "asdeoz-PC";
        // Aquest és el nom de la Base de dades i mai hauria de canviar
        private const string initialCatalog = "FlashCards7";
        private const bool integratedSecurity = true;

        private SqlConnectionStringBuilder builder = new SqlConnectionStringBuilder();
        private DataContext dataContext;

        private void CreaDataContext()
        {
            builder.DataSource = dataSource;
            builder.InitialCatalog = initialCatalog;
            builder.IntegratedSecurity = integratedSecurity;

            dataContext = new DataContext(builder.ConnectionString);
        }



        public Paquet GetPaquet(string paquetId)
        {
            int id = Int32.Parse(paquetId);

            this.CreaDataContext();

            Paquet paquet = (from p in dataContext.GetTable<Paquet>()
                             where p.Id == id
                             select p).First();

            if (paquet != null)
            {
                paquet.Targetes = (from t in dataContext.GetTable<Targeta>()
                                   where t.Paquet == paquet.Id
                                   select t).ToList<Targeta>();
            }

            return paquet;
        }

        public List<Paquet> GetPaquets()
        {

            this.CreaDataContext();

            List<Paquet> llistaP = (from p in dataContext.GetTable<Paquet>()
                                    select p).ToList<Paquet>();

            return llistaP;
        }

        public Rol GetRol(string rolId)
        {
            int id = Int32.Parse(rolId);

            builder.DataSource = dataSource;
            builder.InitialCatalog = initialCatalog;
            builder.IntegratedSecurity = integratedSecurity;

            dataContext = new DataContext(builder.ConnectionString);

            Rol rol = (from r in dataContext.GetTable<Rol>()
                             where r.Id == id
                             select r).First();

            return rol;
        }


        public Usuari LoginUsuari(string sLogin, string sContrasenya)
        {
            this.CreaDataContext();

            Usuari usuari;

            try
            {

                usuari = (from u in dataContext.GetTable<Usuari>()
                          where (u.Login == sLogin) && (u.Contrasenya == sContrasenya)
                          select u).First();


                usuari.RolE = GetRol(usuari.Rol.ToString());

                List<Usuari_Paquet> llUsuariPaquet = (from up in dataContext.GetTable<Usuari_Paquet>()
                                                      where up.Usuari == usuari.Id
                                                      select up).ToList<Usuari_Paquet>();

                usuari.Paquets = new List<Paquet>();

                foreach (Usuari_Paquet up in llUsuariPaquet)
                {
                    Paquet paquet = (from p in dataContext.GetTable<Paquet>()
                                     where p.Id == up.Paquet
                                     select p).First();

                    if (paquet != null)
                    {
                        paquet.Targetes = (from t in dataContext.GetTable<Targeta>()
                                           where t.Paquet == paquet.Id
                                           select t).ToList<Targeta>();

                        usuari.Paquets.Add(paquet);
                    }
                }
            }

            catch
            {
                usuari = null;
            }

            return usuari;

        }

        public List<Usuari> GetUsuaris()
        {
            this.CreaDataContext();

            List<Usuari> usuaris = (from u in dataContext.GetTable<Usuari>()
                                    select u).ToList<Usuari>();

            foreach (Usuari u in usuaris)
            {
                u.RolE = (from r in dataContext.GetTable<Rol>()
                          where r.Id == u.Rol
                          select r).First();
            }

            return usuaris;
        }

        public List<Tematica> GetTematiques()
        {
            this.CreaDataContext();

            List<Tematica> llistaTematiques = (from t in dataContext.GetTable<Tematica>()
                                               select t).ToList<Tematica>();

            foreach (Tematica t in llistaTematiques)
            {
                t.Paquets = new List<Paquet>();

                List<Paquet_Tematica> llistaPaquetTematica = (from pt in dataContext.GetTable<Paquet_Tematica>()
                                                              where pt.Tematica == t.Id
                                                              select pt).ToList<Paquet_Tematica>();

                foreach (Paquet_Tematica pt in llistaPaquetTematica)
                {
                    t.Paquets.Add(GetPaquet(pt.Paquet.ToString()));
                }

            }


            return llistaTematiques;
        }

        public bool InsertDirectTargeta(Targeta t)
        {
            this.CreaDataContext();

            try
            {
                if (t != null)
                {
                    dataContext.GetTable<Targeta>().InsertOnSubmit(t);

                    dataContext.SubmitChanges();
                }
                else
                {
                    return false;
                }
            }
            catch(Exception ex)
            {
                Debug.WriteLine(ex.Message);
                return false;
            }

            return true;
        }

        public bool InsertTargeta(string titolF, string descF, string titolB, string descB, string idPaquet)
        {
            this.CreaDataContext();

            try
            {
                Targeta t = new Targeta();
                t.TitolFront = titolF;
                t.DescFront = descF;
                t.TitolBack = titolB;
                t.DescBack = descB;
                t.Paquet = Int32.Parse(idPaquet);

                if (t != null)
                {
                    dataContext.GetTable<Targeta>().InsertOnSubmit(t);

                    dataContext.SubmitChanges();
                }
                else
                {
                    return false;
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                return false;
            }

            return true;
        }

        public Codis ComprovaValidaCodi(string sCodi)
        {
            this.CreaDataContext();

            try
            {

                Codis oCodi = (from c in dataContext.GetTable<Codis>()
                               where c.Codi == sCodi
                               select c).First<Codis>();
                return oCodi;

            }
            catch
            {
                return null;
            }

        }

        public bool ActualitzaCredit(decimal dCredit, int idUsuari)
        {
            this.CreaDataContext();

            try
            {
                Usuari oUsuari = (from u in dataContext.GetTable<Usuari>()
                                  where u.Id == idUsuari
                                  select u).First<Usuari>();

                oUsuari.Credit += dCredit;

                dataContext.SubmitChanges();

                return true;
            }
            catch
            {
                return false;
            }
        }

        public bool CodiUtilitzat(int idCodi)
        {
            this.CreaDataContext();

            try
            {
                Codis oCodi = (from c in dataContext.GetTable<Codis>()
                                  where c.Id == idCodi
                                  select c).First<Codis>();

                oCodi.Utilitzat = true;

                dataContext.SubmitChanges();

                return true;
            }
            catch
            {
                return false;
            }
        }

        public Usuari GetUsuari(int idUsuari)
        {
            this.CreaDataContext();

            Usuari usuari;

            try
            {

                usuari = (from u in dataContext.GetTable<Usuari>()
                          where u.Id == idUsuari
                          select u).First();


                usuari.RolE = GetRol(usuari.Rol.ToString());

                List<Usuari_Paquet> llUsuariPaquet = (from up in dataContext.GetTable<Usuari_Paquet>()
                                                      where up.Usuari == usuari.Id
                                                      select up).ToList<Usuari_Paquet>();

                usuari.Paquets = new List<Paquet>();

                foreach (Usuari_Paquet up in llUsuariPaquet)
                {
                    Paquet paquet = (from p in dataContext.GetTable<Paquet>()
                                     where p.Id == up.Paquet
                                     select p).First();

                    if (paquet != null)
                    {
                        paquet.Targetes = (from t in dataContext.GetTable<Targeta>()
                                           where t.Paquet == paquet.Id
                                           select t).ToList<Targeta>();

                        usuari.Paquets.Add(paquet);
                    }
                }
            }

            catch
            {
                usuari = null;
            }

            return usuari;
        }

        public bool ComprarPaquet(int idUsuari, int idPaquet)
        {
            this.CreaDataContext();

            try
            {
                Usuari oUsuari = (from u in dataContext.GetTable<Usuari>()
                                  where u.Id == idUsuari
                                  select u).First<Usuari>();

                Paquet oPaquet = (from p in dataContext.GetTable<Paquet>()
                                  where p.Id == idPaquet
                                  select p).First<Paquet>();

                Usuari_Paquet oUsuariPaquet = new Usuari_Paquet();

                oUsuariPaquet.Paquet = oPaquet.Id;
                oUsuariPaquet.Usuari = oUsuari.Id;

                oUsuariPaquet.Data = DateTime.Now;

                if (oUsuari.Credit < oPaquet.Preu)
                {
                    return false;
                }

                oUsuari.Credit -= oPaquet.Preu;

                dataContext.GetTable<Usuari_Paquet>().InsertOnSubmit(oUsuariPaquet);

                dataContext.SubmitChanges();

                return true;
            }
            catch
            {
                return false;
            }
        }

        public bool ComprovaUsuariPaquet(int idUsuari, int idPaquet)
        {
            this.CreaDataContext();

            try
            {

                Usuari_Paquet oUsuariPaquet = (from up in dataContext.GetTable<Usuari_Paquet>()
                                               where (up.Usuari == idUsuari) && (up.Paquet == idPaquet)
                                               select up).First<Usuari_Paquet>();

                if (oUsuariPaquet != null)
                {
                    return true;
                }

                return false;
            }
            catch
            {
                return false;
            }
        }

        public List<Rol> GetRols()
        {
            this.CreaDataContext();

            List<Rol> rols;

            try
            {
                rols = (from r in dataContext.GetTable<Rol>()
                        select r).ToList<Rol>();
            }
            catch
            {
                rols = new List<Rol>();
            }

            return rols;

        }

        public bool InsertOrEditUsuari(Usuari usuari)
        {
            this.CreaDataContext();

            try
            {
                if (usuari != null)
                {
                    if (usuari.Id != 0) // Modificar usuari
                    {
                        Usuari nouUsuari = (from u in dataContext.GetTable<Usuari>()
                                            where u.Id == usuari.Id
                                            select u).First<Usuari>();

                        nouUsuari.Nom = usuari.Nom;
                        nouUsuari.Login = usuari.Login;
                        nouUsuari.Contrasenya = usuari.Contrasenya;
                        nouUsuari.Cognom1 = usuari.Cognom1;
                        nouUsuari.Cognom2 = usuari.Cognom2;
                        nouUsuari.Rol = usuari.RolE.Id;
                        nouUsuari.Email = usuari.Email;
                        nouUsuari.Credit = usuari.Credit;
                    }
                    else // Nou usuari
                    {
                        usuari.DTCreacio = DateTime.Now;
                        usuari.Rol = usuari.RolE.Id;

                        dataContext.GetTable<Usuari>().InsertOnSubmit(usuari);
                    }

                    // Fem els canvis a BBDD
                    dataContext.SubmitChanges();

                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch
            {
                return false;
            }
        }


        public bool DeleteUsuari(int idUsuari)
        {
            this.CreaDataContext();

            try
            {
                Usuari usuari = (from u in dataContext.GetTable<Usuari>()
                                 where u.Id == idUsuari
                                 select u).First<Usuari>();

                dataContext.GetTable<Usuari>().DeleteOnSubmit(usuari);

                dataContext.SubmitChanges();

                return true;
            }
            catch
            {
                return false;
            }
        }
    }
}
